#include <iostream>
#include <cmath>
using namespace std;


class point
{
public:
	double x;
	double y;
	point(double xe, double ye);
	point();
	void scale(double a);
	void add(point & q);
	void print();
};


void mergeSort(point A[], int q, int r);
void Merge(point A[], int p, int q, int r);
void merge(const point L[],int m,const point R[], int n, point C[]);

class rectangle
{
	point l;
	point r;
	bool empty;
public:
	rectangle(point &p, point &q);
	rectangle();
	double area();
	void scale(double a);
	void translate(point & p);
	bool containsPoint(point &p);
	void intersect(rectangle & R);
	void pointSetIntersect(point A[], int n, point B[], int &m);
	void print();
};


int main()
{
	cout<<endl<<"************ Testing  the point class ********************"<<endl;
	point p(5,3); 
	cout<<"p: "; p.print();
	p.scale(3);
	cout<<"p scaled by 3: ";p.print();
	point q(6,10);
	cout<<"q: "; q.print();
	p.add(q);
	cout<<"p = p+q: ";p.print();
    cout<<endl<<"************ Testing  the rectangle class ********************"<<endl;
	cout<<endl<<"------------ Testing  area, translate, scale, containsPoint, and print ----"<<endl;
	rectangle R(q, p);
	cout<<"R: "<<endl; R.print();
	cout<<"Area: "<<R.area()<<endl<<endl;
	R.scale(1.5);cout<<"Scaling by 1.5"<<endl;
	R.print();
	cout<<"Area: "<<R.area()<<endl<<endl;
	R.translate(p);cout<<"Translating by p"<<endl;
	R.print();
	point test(30,49);
	cout<<endl<<"test point: ";test.print();
	cout<<"R contains test point:"<<R.containsPoint(test)<<endl;

	point test2(30,40);
	cout<<endl<<"test point 2: ";test2.print();
	cout<<"R contains test point 2:"<<R.containsPoint(test2)<<endl;

	cout<<endl<<"------------ Testing  intersect --------------------"<<endl;

	cout<<"R:"<<endl;R.print();
	point pnew(29,35), qnew(53,47);
	rectangle Rnew(pnew, qnew);
	cout<<"New rectangle:"<<endl;Rnew.print();
	R.intersect(Rnew);
    cout<<"Intersecting R with new rectangle:"<<endl;R.print();
    
	cout<<endl<<"R:"<<endl;R.print();
	point pnew2(100,100), qnew2(110,110);
	rectangle Rnew2(pnew2, qnew2);
	cout<<"New rectangle 2:"<<endl;Rnew2.print();
	Rnew2.intersect(R);
    cout<<"Intersecting  new rectangle 2 with R:"<<endl;Rnew2.print();

	
	cout<<endl<<"------------ Testing pointSetIntersect --------------------"<<endl;
	
	point B[3], q1(3,4),q2(rand()%100,40),q3(52.5,rand()%100),q4(0,0),q5(35,46),q6(100,100), A[5]={q1,q2,q3,q4,q5};
	cout<<"A:"<<endl;for(int i=0;i<5;i++) A[i].print();
	R.print();
	int m;
	R.pointSetIntersect(A, 5, B, m);
	cout<<"B:"<<endl;for(i=0;i<m;i++) B[i].print();
	cout<<endl<<"m= "<<m<<endl;


	cout<<endl<<"-------------Testing mergeSort -------------------"<<endl;
	
	cout<<"A:"<<endl;for(i=0;i<5;i++) A[i].print();
	mergeSort(A,0,5);
	cout<<"A sorted in order of increasing x-coordinate:"<<endl;for(i=0;i<5;i++) A[i].print();
	
	cout<<endl<<"--------------------------------"<<endl;

	return 0;
}


void merge(const point L[],int m,const point R[], int n, point C[] )
{
   int i =0, j = 0; 
   for(int k = 0;k<m+n;k++) 
   {
      if (i< m &&  j < n) 
          if (L[i].x<R[j].x) 
               { C[k] = L[i];  i++;}  
          else  
              {  C[k] = R[j];   j++;}
      else if  (i==m) {C[k] = R[j]; j = j+1;} 
      else if  (j==n) {C[k] = L[i];  i = i+1;}
  }
}

void Merge(point A[], int p, int q, int r)
{
	point * B = new point[q-p+1];
	point * C= new point[r-q];
	point * D= new point[r-p+1];
	for(int i=p;i<=q;i++)B[i-p]=A[i];
	for(i=q+1;i<=r;i++) C[i-q-1]=A[i];
	merge(B,q-p+1,C, r-q, D);
	for(i=p;i<=r;i++) A[i]=D[i-p];
	delete []B; delete []C; delete [] D;
}

void mergeSort(point A[], int p, int r)
{
	if(p<r)
	{
		int q=(p+r)/2;
		mergeSort(A, p, q);
		mergeSort(A, q+1, r);
		Merge(A, p, q, r);
	}
}


point::point(double xe, double ye)
{
	x=xe;
	y=ye;
}

point::point()
{
	x=0;
	y=0;
}

void point::scale(double a)
{
	x*=a;
	y*=a;
}

void point::add(point & q)
{
	x+=q.x;
	y+=q.y;
}


void point::print()
{
	cout<<"("<<x<<","<<y<<")"<<endl;
}

rectangle::rectangle(point &p, point &q)
{
	if(p.x>q.x){ 
		r=p; 
		l=q; 
	}
	else{ 
		r=q; 
		l=p;
	}

	if(l.y>r.y)
		empty = true;
	else 
		empty = false;
}

rectangle::rectangle()
{
	l.x=0; l.y=0;
	r.x=5; r.y=5;
	empty = false;
}

double rectangle::area()
{
	return (r.x-l.x)*(r.y-l.y);
}

void rectangle::scale(double a)
{
	l.x=a*l.x;
	r.x=a*r.x;
	l.y=a*l.y;
	r.y=a*r.y;
}
void rectangle::translate(point &p)
{
	l.x+=p.x;
	l.y+=p.y;
	r.x+=p.x;
	r.y+=p.y;
}
bool rectangle::containsPoint(point &p)
{
	if(((p.x<=r.x) && (p.x>=l.x)) && ((p.y<=r.y) && (p.y>=l.y)))
		return true;
	else
		return false;
}


double max(double a, double b) {if(a>b) return a; else return b;} 
double min(double a, double b) {return - max(-a,-b);} 

void rectangle::intersect(rectangle &R)
{

	l.x = max(R.l.x,l.x);
	l.y = max(R.l.y,l.y);
	r.x = min(R.r.x,r.x);
	r.y = min(R.r.y,r.y);
    if (l.x > r.x ||  l.y > r.y) 
		empty = true;

}

void rectangle::pointSetIntersect(point A[], int n, point B[], int &m)
{
	int j=0;
	for(int i=0;i<n;i++) 
		if(containsPoint(A[i])) 
		{
			B[j]=A[i];
            j++;
		}
	m = j;

}
void rectangle::print()
{
	if(empty) 
		cout <<"   rectangle empty !"<<endl; 
	else 
	{ 
	  cout<<"   left corner:"; 
	  l.print();
	  cout<<"   right corner:";
	  r.print();     
	}
}




